@@ -5,9 +5,8 @@ import threading
 import warnings
 import weakref
 from contextlib import contextmanager, suppress
-from functools import lru_cache, partial, reduce
+from functools import lru_cache, reduce
 from inspect import Parameter, Signature, isclass
-from types import MethodType
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -18,6 +17,7 @@ from typing import (
     Iterator,
     NoReturn,
     Type,
+    TypeVar,
     Union,
     cast,
     get_type_hints,
@@ -25,25 +25,38 @@ from typing import (
 )
 
 from mypy_extensions import mypyc_attr
-from typing_extensions import Protocol, get_args, get_origin
+from typing_extensions import get_args, get_origin
+
+from psygnal._weak_callback import (
+    WeakCallback,
+    _WeakSetattr,
+    _WeakSetitem,
+    weak_callback,
+)
 
 if TYPE_CHECKING:
-    from typing_extensions import Literal, TypeGuard
+    from typing_extensions import Literal
+
+    from ._group import EmissionInfo
+    from ._weak_callback import RefErrorChoice
 
     ReducerFunc = Callable[[tuple, tuple], tuple]
 
 __all__ = ["Signal", "SignalInstance", "_compiled"]
 _NULL = object()
+F = TypeVar("F", bound=Callable)
 
 
 class EmitLoopError(Exception):
-    def __init__(self, slot: Callable, args: tuple, exc: BaseException) -> None:
-        self.slot = slot
+    """Error type raised when an exception occurs during a callback."""
+
+    def __init__(self, slot_repr: str, args: tuple, exc: BaseException) -> None:
+        self.slot_repr = slot_repr
         self.args = args
         self.__cause__ = exc  # mypyc doesn't set this, but uncompiled code would
         super().__init__(
-            f"calling {self.slot} with args={args!r} caused "
-            f"{type(exc).__name__} in emit loop."
+            f"calling {self.slot_repr} with args={args!r} caused "
+            f"{type(exc).__name__}: {exc}."
         )
 
 
@@ -53,7 +66,7 @@ class Signal:
     This is class implements the [descriptor
     protocol](https://docs.python.org/3/howto/descriptor.html#descriptorhowto)
     and is designed to be used as a class attribute, with the supported signature types
-    provided in the contructor:
+    provided in the constructor:
 
     ```python
     from psygnal import Signal
@@ -255,7 +268,7 @@ class SignalInstance:
     signature : Optional[inspect.Signature]
         The signature that this signal accepts and will emit, by default `Signature()`.
     instance : Optional[Any]
-        An object to which this signal is bound. Normally this will be provied by the
+        An object to which this signal is bound. Normally this will be provided by the
         `Signal.__get__` method (see above).  However, an unbound `SignalInstance`
         may also be created directly. by default `None`.
     name : Optional[str]
@@ -279,6 +292,7 @@ class SignalInstance:
 
     _is_blocked: bool = False
     _is_paused: bool = False
+    _debug_hook: ClassVar[Callable[[EmissionInfo], None] | None] = None
 
     def __init__(
         self,
@@ -312,7 +326,7 @@ class SignalInstance:
         self._signature = signature
         self._check_nargs_on_connect = check_nargs_on_connect
         self._check_types_on_connect = check_types_on_connect
-        self._slots: list[SlotCaller] = []
+        self._slots: list[WeakCallback] = []
         self._is_blocked: bool = False
         self._is_paused: bool = False
         self._lock = threading.RLock()
@@ -346,30 +360,33 @@ class SignalInstance:
         check_types: bool | None = ...,
         unique: bool | str = ...,
         max_args: int | None = None,
-    ) -> Callable[[Callable], Callable]:
+        on_ref_error: RefErrorChoice = ...,
+    ) -> Callable[[F], F]:
         ...  # pragma: no cover
 
     @overload
     def connect(
         self,
-        slot: Callable,
+        slot: F,
         *,
         check_nargs: bool | None = ...,
         check_types: bool | None = ...,
         unique: bool | str = ...,
         max_args: int | None = None,
-    ) -> Callable:
+        on_ref_error: RefErrorChoice = ...,
+    ) -> F:
         ...  # pragma: no cover
 
     def connect(
         self,
-        slot: Callable | None = None,
+        slot: F | None = None,
         *,
         check_nargs: bool | None = None,
         check_types: bool | None = None,
         unique: bool | str = False,
         max_args: int | None = None,
-    ) -> Callable[[Callable], Callable] | Callable:
+        on_ref_error: RefErrorChoice = "warn",
+    ) -> Callable[[F], F] | F:
         """Connect a callback (`slot`) to this signal.
 
         `slot` is compatible if:
@@ -411,6 +428,11 @@ class SignalInstance:
             If provided, `slot` will be called with no more more than `max_args` when
             this SignalInstance is emitted.  (regardless of how many arguments are
             emitted).
+        on_ref_error : {'raise', 'warn', 'ignore'}, optional
+            What to do if a weak reference cannot be created.  If 'raise', a
+            ReferenceError will be raised.  If 'warn' (default), a warning will be
+            issued and a strong-reference will be used. If 'ignore' a strong-reference
+            will be used (silently).
 
         Raises
         ------
@@ -427,7 +449,11 @@ class SignalInstance:
         if check_types is None:
             check_types = self._check_types_on_connect
 
-        def _wrapper(slot: Callable, max_args: int | None = max_args) -> Callable:
+        def _wrapper(
+            slot: F,
+            max_args: int | None = max_args,
+            _on_ref_err: RefErrorChoice = on_ref_error,
+        ) -> F:
             if not callable(slot):
                 raise TypeError(f"Cannot connect to non-callable object: {slot}")
 
@@ -440,26 +466,56 @@ class SignalInstance:
                         )
                     return slot
 
-                slot_sig = None
+                slot_sig: Signature | None = None
                 if check_nargs and (max_args is None):
-                    slot_sig, max_args = self._check_nargs(slot, self.signature)
+                    slot_sig, max_args, isqt = self._check_nargs(slot, self.signature)
+                    if isqt:
+                        _on_ref_err = "ignore"
                 if check_types:
                     slot_sig = slot_sig or signature(slot)
                     if not _parameter_types_match(slot, self.signature, slot_sig):
                         extra = f"- Slot types {slot_sig} do not match types in signal."
                         self._raise_connection_error(slot, extra)
 
-                self._slots.append(_slot_caller(slot, max_args))
+                # this type ignore could be fixed with ParamSpec, but that's not yet
+                # supported by mypyc.  we need cb to be a WeakCallback[R], but we can't
+                # preserve the full typing information of the callback without using
+                # Callable[ParamSpec, R] or a general TypeVar('F', bound=Callable).
+                cb = weak_callback(  # type: ignore [var-annotated]
+                    slot,
+                    max_args=max_args,
+                    finalize=self._try_discard,
+                    on_ref_error=_on_ref_err,
+                )
+                self._slots.append(cb)
             return slot
 
         return _wrapper if slot is None else _wrapper(slot)
 
+    def _try_discard(self, callback: WeakCallback, missing_ok: bool = True) -> None:
+        """Try to discard a callback from the list of slots.
+
+        Parameters
+        ----------
+        callback : WeakCallback
+            A callback to discard.
+        missing_ok : bool, optional
+            If `True`, do not raise an error if the callback is not found in the list.
+        """
+        try:
+            self._slots.remove(callback)
+        except ValueError:
+            if not missing_ok:
+                raise
+
     def connect_setattr(
         self,
         obj: weakref.ref | object,
         attr: str,
         maxargs: int | None = None,
-    ) -> _SetattrCaller:
+        *,
+        on_ref_error: RefErrorChoice = "warn",
+    ) -> WeakCallback[None]:
         """Bind an object attribute to the emitted value of this signal.
 
         Equivalent to calling `self.connect(functools.partial(setattr, obj, attr))`,
@@ -471,12 +527,17 @@ class SignalInstance:
         Parameters
         ----------
         obj : Union[weakref.ref, object]
-            An object or weak reference to an object.
+            An object or weak reference (deprecated) to an object.
         attr : str
             The name of an attribute on `obj` that should be set to the value of this
             signal when emitted.
         maxargs : Optional[int]
             max number of positional args to accept
+        on_ref_error: {'raise', 'warn', 'ignore'}, optional
+            What to do if a weak reference cannot be created.  If 'raise', a
+            ReferenceError will be raised.  If 'warn' (default), a warning will be
+            issued and a strong-reference will be used. If 'ignore' a strong-reference
+            will be used (silently).
 
         Returns
         -------
@@ -505,13 +566,26 @@ class SignalInstance:
         >>> t.sig.emit(5)
         >>> assert my_obj.x == 5
         """
-        ref = _get_ref_or_warn(obj, meth="disconnect_setattr")
-
-        if not hasattr(ref(), attr):
-            raise AttributeError(f"Object {ref()} has no attribute {attr!r}")
+        if isinstance(obj, weakref.ReferenceType):  # pragma: no cover
+            warnings.warn(
+                'Using a weakref as the "obj" argument is deprecated. '
+                "Use the object directly instead. This will raise an error in "
+                "a future release.",
+                FutureWarning,
+                stacklevel=2,
+            )
+            obj = obj()
+        if not hasattr(obj, attr):
+            raise AttributeError(f"Object {obj} has no attribute {attr!r}")
 
         with self._lock:
-            caller = _SetattrCaller(ref, attr, maxargs)
+            caller = _WeakSetattr(
+                obj,
+                attr,
+                max_args=maxargs,
+                finalize=self._try_discard,
+                on_ref_error=on_ref_error,
+            )
             self._slots.append(caller)
         return caller
 
@@ -538,24 +612,17 @@ class SignalInstance:
         """
         # sourcery skip: merge-nested-ifs, use-next
         with self._lock:
-            idx = None
-            for i, slot in enumerate(self._slots):
-                if isinstance(slot, _SetattrCaller):
-                    if slot._ref() is obj and slot._attr == attr:
-                        idx = i
-                        break
-
-            if idx is not None:
-                self._slots.pop(idx)
-            elif not missing_ok:
-                raise ValueError(f"No attribute setter connected for {obj}.{attr}")
+            cb = _WeakSetattr(obj, attr, on_ref_error="ignore")
+            self._try_discard(cb, missing_ok)
 
     def connect_setitem(
         self,
         obj: weakref.ref | object,
         key: str,
         maxargs: int | None = None,
-    ) -> _SetitemCaller:
+        *,
+        on_ref_error: RefErrorChoice = "warn",
+    ) -> WeakCallback[None]:
         """Bind a container item (such as a dict key) to emitted value of this signal.
 
         Equivalent to calling `self.connect(functools.partial(obj.__setitem__, attr))`,
@@ -567,12 +634,17 @@ class SignalInstance:
         Parameters
         ----------
         obj : Union[weakref.ref, object]
-            An object or weak reference to an object.
+            An object or weak reference (deprecated) to an object.
         key : str
             Name of the key in `obj` that should be set to the value of this
             signal when emitted
         maxargs : Optional[int]
             max number of positional args to accept
+        on_ref_error: {'raise', 'warn', 'ignore'}, optional
+            What to do if a weak reference cannot be created.  If 'raise', a
+            ReferenceError will be raised.  If 'warn' (default), a warning will be
+            issued and a strong-reference will be used. If 'ignore' a strong-reference
+            will be used (silently).
 
         Returns
         -------
@@ -598,14 +670,28 @@ class SignalInstance:
         >>> t.sig.emit(5)
         >>> assert my_obj == {'x': 5}
         """
-        ref = _get_ref_or_warn(obj, meth="disconnect_setitem")
-
-        if not hasattr(ref(), "__setitem__"):
-            raise TypeError(f"Object {ref()} does not support __setitem__")
+        if isinstance(obj, weakref.ReferenceType):  # pragma: no cover
+            warnings.warn(
+                'Using a weakref as the "obj" argument is deprecated. '
+                "Use the object directly instead. This will raise an error in "
+                "a future release.",
+                FutureWarning,
+                stacklevel=2,
+            )
+            obj = obj()
+        if not hasattr(obj, "__setitem__"):
+            raise TypeError(f"Object {obj} does not support __setitem__")
 
         with self._lock:
-            caller = _SetitemCaller(ref, key, maxargs)
+            caller = _WeakSetitem(
+                obj,  # type: ignore
+                key,
+                max_args=maxargs,
+                finalize=self._try_discard,
+                on_ref_error=on_ref_error,
+            )
             self._slots.append(caller)
+
         return caller
 
     def disconnect_setitem(
@@ -629,25 +715,29 @@ class SignalInstance:
         ValueError
             If `missing_ok` is `True` and no item setter is connected.
         """
+        if not hasattr(obj, "__setitem__"):
+            raise TypeError(f"Object {obj} does not support __setitem__")
+
         # sourcery skip: merge-nested-ifs, use-next
         with self._lock:
-            idx = None
-            for i, slot in enumerate(self._slots):
-                if isinstance(slot, _SetitemCaller):
-                    if slot._ref() is obj and slot._key == key:
-                        idx = i
-                        break
-            if idx is not None:
-                self._slots.pop(idx)
-            elif not missing_ok:
-                raise ValueError(f"No item setter connected for {obj}.{key}")
+            caller = _WeakSetitem(obj, key, on_ref_error="ignore")
+            self._try_discard(caller, missing_ok)
 
     def _check_nargs(
         self, slot: Callable, spec: Signature
-    ) -> tuple[Signature | None, int | None]:
+    ) -> tuple[Signature | None, int | None, bool]:
         """Make sure slot is compatible with signature.
 
         Also returns the maximum number of arguments that we can pass to the slot
+
+        Returns
+        -------
+        slot_sig : Signature | None
+            The signature of the slot, or None if it could not be determined.
+        maxargs : int | None
+            The maximum number of arguments that we can pass to the slot.
+        is_qt : bool
+            Whether the slot is a Qt slot.
         """
         try:
             slot_sig = _get_signature_possibly_qt(slot)
@@ -655,7 +745,7 @@ class SignalInstance:
             warnings.warn(
                 f"{e}. To silence this warning, connect with " "`check_nargs=False`"
             )
-            return None, None
+            return None, None, False
         minargs, maxargs = _acceptable_posarg_range(slot_sig)
 
         n_spec_params = len(spec.parameters)
@@ -666,8 +756,8 @@ class SignalInstance:
                 f"arguments, but spec only provides {n_spec_params}"
             )
             self._raise_connection_error(slot, extra)
-        _sig = None if isinstance(slot_sig, str) else slot_sig
-        return _sig, maxargs
+
+        return None if isinstance(slot_sig, str) else slot_sig, maxargs, True
 
     def _raise_connection_error(self, slot: Callable, extra: str = "") -> NoReturn:
         name = getattr(slot, "__name__", str(slot))
@@ -679,7 +769,9 @@ class SignalInstance:
     def _slot_index(self, slot: Callable) -> int:
         """Get index of `slot` in `self._slots`.  Return -1 if not connected."""
         with self._lock:
-            normed = _slot_caller(slot)
+            normed = weak_callback(slot, on_ref_error="ignore")
+            # NOTE:
+            # the == method here relies on the __eq__ method of each SlotCaller subclass
             return next((i for i, s in enumerate(self._slots) if s == normed), -1)
 
     def disconnect(self, slot: Callable | None = None, missing_ok: bool = True) -> None:
@@ -708,9 +800,6 @@ class SignalInstance:
             idx = self._slot_index(slot)
             if idx != -1:
                 self._slots.pop(idx)
-                _PARTIAL_CACHE.pop(id(slot), None)
-                if isinstance(slot, _PartialMethodCaller):
-                    _PARTIAL_CACHE.pop(slot._slot_id, None)
             elif not missing_ok:
                 raise ValueError(f"slot is not connected: {slot}")
 
@@ -762,7 +851,7 @@ class SignalInstance:
             These arguments will be passed when calling each slot (unless the slot
             accepts fewer arguments, in which case extra args will be discarded.)
         check_nargs : Optional[bool]
-            If `False` and the provided arguments cannot be successfuly bound to the
+            If `False` and the provided arguments cannot be successfully bound to the
             signature of this Signal, raise `TypeError`.  Incurs some overhead.
             by default False.
         check_types : Optional[bool]
@@ -803,6 +892,11 @@ class SignalInstance:
             self._args_queue.append(args)
             return None
 
+        if SignalInstance._debug_hook is not None:
+            from ._group import EmissionInfo
+
+            SignalInstance._debug_hook(EmissionInfo(self, args))
+
         if asynchronous:
             sd = EmitThread(self, args)
             sd.start()
@@ -848,21 +942,16 @@ class SignalInstance:
         )
 
     def _run_emit_loop(self, args: tuple[Any, ...]) -> None:
-        rem: list[SlotCaller] = []
         # allow receiver to query sender with Signal.current_emitter()
         with self._lock:
             with Signal._emitting(self):
                 for caller in self._slots:
                     try:
-                        if caller(args):
-                            # if the slotcaller returns `True`, the object weakref is
-                            # dead and needs to be disconnected
-                            rem.append(caller)
+                        caller.cb(args)
                     except Exception as e:
-                        raise EmitLoopError(slot=caller.slot(), args=args, exc=e) from e
-
-            for slot in rem:
-                self.disconnect(slot)
+                        raise EmitLoopError(
+                            slot_repr=repr(caller), args=args, exc=e
+                        ) from e
 
         return None
 
@@ -924,7 +1013,7 @@ class SignalInstance:
             follows: `self.emit(*functools.reduce(reducer, [(1,), (1,), (1,)]))`
 
         initial: any, optional
-            intial value to pass to `functools.reduce`
+            initial value to pass to `functools.reduce`
 
         Examples
         --------
@@ -957,7 +1046,7 @@ class SignalInstance:
     def paused(
         self, reducer: ReducerFunc | None = None, initial: Any = _NULL
     ) -> ContextManager[None]:
-        """Context manager to temporarly pause this signal.
+        """Context manager to temporarily pause this signal.
 
         Parameters
         ----------
@@ -970,7 +1059,7 @@ class SignalInstance:
             For example, three `emit(1)` events would be reduced and re-emitted as
             follows: `self.emit(*functools.reduce(reducer, [(1,), (1,), (1,)]))`
         initial: any, optional
-            intial value to pass to `functools.reduce`
+            initial value to pass to `functools.reduce`
 
         Examples
         --------
@@ -1057,20 +1146,6 @@ class EmitThread(threading.Thread):
 
 # #############################################################################
 # #############################################################################
-class PartialMethod(Protocol):
-    """Bound method wrapped in partial: `partial(MyClass().some_method, y=1)`."""
-
-    func: MethodType
-    args: tuple
-    keywords: dict[str, Any]
-
-    def __call__(self, *args: Any, **kwds: Any) -> Any:
-        ...
-
-
-def _is_partial_method(obj: object) -> TypeGuard[PartialMethod]:
-    """Return `True` of `obj` is a `functools.partial` wrapping a bound method."""
-    return isinstance(obj, partial) and isinstance(obj.func, MethodType)
 
 
 def signature(obj: Any) -> inspect.Signature:
@@ -1127,206 +1202,6 @@ def _build_signature(*types: type[Any]) -> Signature:
     return Signature(params)
 
 
-class SlotCaller:
-    """ABC for a "stored" slot.
-
-    !!! note
-
-        We're not using a real ABC here because PySide is doing some weird stuff
-        that causes mypyc to complain with:
-
-            src/psygnal/_signal.py:1108: in <module>
-                class SlotCaller(ABC):
-            E   TypeError: mypyc classes can't have a metaclass
-
-        ...but *only* when PySide2 is imported (not with PyQt5).
-
-    A SlotCaller is responsible for actually calling a stored slot during the
-    `run_emit_loop`.  It is used to allow for different types of slots to be stored
-    in a `SignalInstance` (such as a function, a bound method, a partial to a bound
-    method), while still allowing them to be called in the same way.
-
-    The main reason is that some slot types need to derefence a weakref during
-    call time, while others don't.
-    """
-
-    def __call__(self, args: tuple[object, ...]) -> bool:
-        """Call the slot, return True if the object ref is dead and needs cleaning."""
-        raise NotImplementedError()
-
-    def __eq__(self, other: object) -> bool:
-        """Return True if `other` is equal to this SlotCaller."""
-        raise NotImplementedError()
-
-    def slot(self) -> Callable:
-        """Reconstruct the original slot."""
-        raise NotImplementedError()
-
-
-def _slot_caller(slot: Callable, max_args: int | None = None) -> SlotCaller:
-    """Factory function to return a `SlotCaller` appropriate for the given slot."""
-    if isinstance(slot, SlotCaller):
-        return slot
-    if isinstance(slot, MethodType):
-        return _BoundMethodCaller(slot, max_args)
-    if _is_partial_method(slot):
-        _id = id(slot)
-        if _id not in _PARTIAL_CACHE:
-            _PARTIAL_CACHE[_id] = _PartialMethodCaller(slot, max_args)
-        return _PARTIAL_CACHE[_id]
-    return _FunctionCaller(slot, max_args)
-
-
-class _FunctionCaller(SlotCaller):
-    """Simple caller of a plain function."""
-
-    def __init__(self, slot: Callable, max_args: int | None = None) -> None:
-        self._slot = slot
-        self._max_args = max_args
-
-    def __call__(self, args: tuple[object, ...]) -> bool:
-        if self._max_args is not None:
-            args = args[: self._max_args]
-        self._slot(*args)
-        return False
-
-    def __eq__(self, other: object) -> bool:
-        return isinstance(other, _FunctionCaller) and self._slot == other._slot
-
-    def slot(self) -> Callable:
-        return self._slot
-
-
-class _SetattrCaller(SlotCaller):
-    """Caller to set an attribute on an object."""
-
-    def __init__(
-        self, ref: weakref.ReferenceType, attr: str, max_args: int | None = None
-    ) -> None:
-        self._ref = ref
-        self._attr = attr
-        self._max_args = max_args
-
-    def __call__(self, args: tuple[object, ...]) -> bool:
-        obj = self._ref()
-        if obj is None:
-            return True
-        if self._max_args is not None:
-            args = args[: self._max_args]
-        setattr(obj, self._attr, args[0] if len(args) == 1 else args)
-        return False
-
-    def __eq__(self, other: object) -> bool:
-        return (
-            isinstance(other, _SetattrCaller)
-            and self._ref == other._ref
-            and self._attr == other._attr
-        )
-
-    def slot(self) -> Callable:
-        return partial(setattr, self._ref(), self._attr)
-
-
-class _SetitemCaller(SlotCaller):
-    """Caller to call __setitem__ on an object."""
-
-    def __init__(
-        self, ref: weakref.ReferenceType, key: Any, max_args: int | None = None
-    ) -> None:
-        self._ref = ref
-        self._key = key
-        self._max_args = max_args
-
-    def __call__(self, args: tuple[object, ...]) -> bool:
-        obj = self._ref()
-        if obj is None:
-            return True
-        if self._max_args is not None:
-            args = args[: self._max_args]
-        obj[self._key] = args[0] if len(args) == 1 else args
-        return False
-
-    def __eq__(self, other: object) -> bool:
-        return (
-            isinstance(other, _SetitemCaller)
-            and self._ref == other._ref
-            and self._key == other._key
-        )
-
-    def slot(self) -> Callable:
-        obj = self._ref()
-        return partial(obj.__setitem__, self._key)  # type: ignore
-
-
-class _BoundMethodCaller(SlotCaller):
-    """Caller of a (dereferenced) bound method."""
-
-    def __init__(self, slot: MethodType, max_args: int | None = None) -> None:
-        self._ref, self._method_name = _get_method_name(slot)
-        self._max_args = max_args
-
-    def __call__(self, args: tuple[object, ...]) -> bool:
-        obj = self._ref()
-        if obj is None:
-            return True
-        method = getattr(obj, self._method_name, None)
-        if method is None:
-            return True  # object has changed?
-        if self._max_args is not None:
-            args = args[: self._max_args]
-        method(*args)
-        return False
-
-    def __eq__(self, other: object) -> bool:
-        return (
-            isinstance(other, _BoundMethodCaller)
-            and self._ref == other._ref
-            and self._method_name == other._method_name
-        )
-
-    def slot(self) -> MethodType:
-        obj = self._ref()
-        if obj is None:
-            raise RuntimeError("object has been deleted")  # pragma: no cover
-        return cast(MethodType, getattr(obj, self._method_name))
-
-
-class _PartialMethodCaller(SlotCaller):
-    """Caller of a partial to a (dereferenced) bound method."""
-
-    def __init__(self, slot: PartialMethod, max_args: int | None = None) -> None:
-        self._ref, self._method_name = _get_method_name(slot.func)
-        self._max_args = max_args
-        self._partial_args = slot.args
-        self._partial_kwargs = slot.keywords
-        self._slot_id = id(slot)
-
-    def __call__(self, args: tuple[object, ...]) -> bool:
-        obj = self._ref()
-        if obj is None:
-            return True
-        if self._max_args is not None:
-            args = args[: self._max_args]
-        method = getattr(obj, self._method_name)
-        method(*self._partial_args, *args, **self._partial_kwargs)
-        return False
-
-    def __eq__(self, other: object) -> bool:
-        return (
-            isinstance(other, _PartialMethodCaller)
-            and self._ref == other._ref
-            and self._method_name == other._method_name
-        )
-
-    def slot(self) -> PartialMethod:
-        obj = self._ref()
-        if obj is None:
-            raise RuntimeError("object has been deleted")  # pragma: no cover
-        method = getattr(obj, self._method_name)
-        _partial = partial(method, *self._partial_args, **self._partial_kwargs)
-        return cast(PartialMethod, _partial)
-
-
 # def f(a, /, b, c=None, *d, f=None, **g): print(locals())
 #
 # a: kind=POSITIONAL_ONLY,       default=Parameter.empty    # 1 required posarg
@@ -1443,38 +1318,6 @@ def _is_subclass(left: type[Any], right: type) -> bool:
     return issubclass(left, right)
 
 
-_PARTIAL_CACHE: dict[int, _PartialMethodCaller] = {}
-
-
-def _prune_partial_cache() -> None:
-    """Remove any partial methods whose object has been garbage collected."""
-    for key, caller in list(_PARTIAL_CACHE.items()):
-        if caller._ref() is None:
-            del _PARTIAL_CACHE[key]
-
-
-def _get_method_name(slot: MethodType) -> tuple[weakref.ref, str]:
-    obj = slot.__self__
-    # some decorators will alter method.__name__, so that obj.method
-    # will not be equal to getattr(obj, obj.method.__name__).
-    # We check for that case here and find the proper name in the function's closures
-    if getattr(obj, slot.__name__, None) != slot:
-        for c in slot.__closure__ or ():
-            cname = getattr(c.cell_contents, "__name__", None)
-            if cname and getattr(obj, cname, None) == slot:
-                return weakref.ref(obj), cname
-        # slower, but catches cases like assigned functions
-        # that won't have function in closure
-        for name in reversed(dir(obj)):  # most dunder methods come first
-            if getattr(obj, name) == slot:
-                return weakref.ref(obj), name
-        # we don't know what to do here.
-        raise RuntimeError(  # pragma: no cover
-            f"Could not find method on {obj} corresponding to decorated function {slot}"
-        )
-    return weakref.ref(obj), slot.__name__
-
-
 def _guess_qtsignal_signature(obj: Any) -> str | None:
     """Return string signature if `obj` is a SignalInstance or Qt emit method.
 
@@ -1513,20 +1356,6 @@ def _ridiculously_call_emit(emitter: Any) -> str | None:
     return None  # pragma: no cover
 
 
-def _get_ref_or_warn(obj: object, meth: str) -> weakref.ReferenceType:
-    try:
-        return obj if isinstance(obj, weakref.ref) else weakref.ref(obj)
-    except TypeError:  # pragma: no cover
-        import warnings
-
-        warnings.warn(
-            f"Could not create weakref to `{obj}`. Call `{meth}` "
-            "manually upon object deletion to avoid memory leaks.",
-            RuntimeWarning,
-        )
-        return lambda: obj  # type: ignore [return-value]
-
-
 _compiled: bool
 
 
